package br.com.abellabank.terminal.entidades;

import br.com.abellabank.terminal.exceptions.SaldoInsuficienteException;
import br.com.abellabank.terminal.exceptions.ValorInvalidoException;

public abstract class Conta {

	private int numeroAgencia;
	private int numeroConta;
	private String titular;
	private double saldo;
	
	private static final double VALOR_DEPOSITO_MINIMO = 0;
	public static final double VALOR_COTACAO_EURO = 6;
	public static final double VALOR_COTACAO_DOLAR = 5;

	public Conta(int numeroAgencia, int numeroConta, String titular) {
		this.numeroAgencia = numeroAgencia;
		this.numeroConta = numeroConta;
		this.titular = titular;
	}

	public Conta(int numeroAgencia, int numeroConta, String titular, double saldo) {
		this.numeroAgencia = numeroAgencia;
		this.numeroConta = numeroConta;
		this.titular = titular;
		this.saldo = saldo;
	}

	public final void depositar(double montante) throws ValorInvalidoException {

		if(montante <= 0) {
			throw new ValorInvalidoException("O valor tem que ser superior a 0, vc colocou " + montante);
		}
		
		if (montante > VALOR_DEPOSITO_MINIMO) {
			// this.saldo = this.saldo + montante;
			this.saldo += montante;
		}
	}
	
	public final void depositar(double montante, TipoMoeda moeda) throws ValorInvalidoException  {

		double saldoDepositar = 0;
		
		if(montante <= 0) {
			throw new ValorInvalidoException("O valor tem que ser superior a 0, vc colocou " + montante);
		}
		
		if(moeda.equals(TipoMoeda.REAL)) {
			saldoDepositar = montante;
		} else if (moeda.equals(TipoMoeda.DOLAR)) {
			saldoDepositar = montante * VALOR_COTACAO_DOLAR;
		}else if (moeda.equals(TipoMoeda.EURO)) {
			saldoDepositar = montante * VALOR_COTACAO_EURO;
		}
		
		this.depositar(saldoDepositar);
	}

	public void sacar(double montante) throws ValorInvalidoException, SaldoInsuficienteException {
		
		if(montante <= 0) {
			throw new ValorInvalidoException("O valor tem que ser superior a 0, vc colocou " + montante);
		}
		
		if (montante > VALOR_DEPOSITO_MINIMO && montante <= this.saldo) {
			// this.saldo = this.saldo - montante;
			this.saldo -= montante;
		} else {
			throw new SaldoInsuficienteException("Vc colocou valor menor que " + VALOR_DEPOSITO_MINIMO + " ou valor superior ao saldo");
		}
	}

	public double getExtrato() {
		return this.saldo;
	}

	public int getNumeroAgencia() {
		return numeroAgencia;
	}

	public void setNumeroAgencia(int numeroAgencia) {
		this.numeroAgencia = numeroAgencia;
	}

	public int getNumeroConta() {
		return numeroConta;
	}

	public void setNumeroConta(int numeroConta) {
		this.numeroConta = numeroConta;
	}

	public String getTitular() {
		return titular;
	}

	public void setTitular(String titular) {
		this.titular = titular;
	}

	@Override
	public String toString() {
		return "numeroAgencia=" + numeroAgencia + ", numeroConta=" + numeroConta + ", titular=" + titular;
	}
	
	

}
